package com.example.groracle.Brokers;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
/**
 * Class representing base for database operations
 * @author tdosek
 * @version 1.0
 */
public class CommonDatabaseBroker {
    private String sqlStatement = null;
    private ArrayList<HashMap<String,Object>> resultList = null; 
    private String url = null;
    private String user = null;
    private String password = null;
    /**
     * Constructor of the class
     * @param sqlStatement String representation of initial SQL statement. It can be null
     * and get set by relevant setter later
     */
    public CommonDatabaseBroker(String sqlStatement){
	this.sqlStatement = sqlStatement;
    }
    /**
     * Method for execution of Insert, Update or Delete statements.
     * No result set is returned by this one.
     */
    public void executeInsertUpdateDelete(){
	try {
	    Class.forName("org.postgresql.Driver");
	} catch (ClassNotFoundException e) {
	    System.out.println("Failed to load jdbc driver");
	    e.printStackTrace();
	}
    	try {
    	    loadDbConnectionInfo();
    	    // 	Initializing variables for DB connection
    	    Connection con = DriverManager.getConnection(url, user, password);
    	    Statement st = con.createStatement();
    	    st.executeUpdate(sqlStatement);
    	} catch (SQLException e) {
    	    System.out.println("Failed to execute SQL query");
    	    e.printStackTrace();
	}   		
    } 
    /**
     * Method for execution of Select statement and generally statements that return result sets
     */
    public void executeSelect(){
	// Loading the driver
	try {
	    Class.forName("org.postgresql.Driver");
	} catch (ClassNotFoundException e) {
	    System.out.println("Failed to load jdbc driver");
	    e.printStackTrace();
	}
	loadDbConnectionInfo();
	// Initializing variables for DB connection
	Connection con = null;
	Statement st = null;
	ResultSet rs = null;
	try {
	    con = DriverManager.getConnection(url, user, password);
	    st = con.createStatement();
	    rs = st.executeQuery(sqlStatement);
	    resultList = resultSetToList(rs);
	}catch (SQLException e){
	    System.out.println("SQL exception occured");
   	    e.printStackTrace();
	}finally{
	    try {
		if (rs != null){
		    rs.close();
		}
		if (st != null){
		    st.close();
		}
		if (con != null){
		    con.close();
		}
	    }catch (SQLException e) {
		System.out.println("Closing connections failed, they "
			+ "probably do not exist anymore");
	    }
	}	
    }
    /**
     * Method that serves for parsing result sets
     * @param rs Result set that is to be parsed
     * @return Abstract object that represents the result set
     * @throws SQLException
     */
    private ArrayList<HashMap<String, Object>> resultSetToList(ResultSet rs) throws SQLException{
	ResultSetMetaData md = rs.getMetaData();
	int columns = md.getColumnCount();
	ArrayList<HashMap<String, Object>> list = new ArrayList<HashMap<String, Object>>(50);
	while (rs.next()){
	    HashMap<String, Object> row = new HashMap<String, Object>(columns);
	    for(int i=1; i<=columns; ++i){
		row.put(md.getColumnName(i),rs.getObject(i));
	    }
	    list.add(row);
	}
	return list;
    }
    /**
     * Method for loading connection information from config file
     */
    private void loadDbConnectionInfo(){
	// Reading actual DB configuration
	ConfigLoader config = new ConfigLoader();
	config.loadConfig();
	url = "jdbc:postgresql://" + config.getDbHost() + ":" + config.getDbPort() + "/" + config.getDbInstance();
	user = config.getDbUserName();
	password = config.getDbUserPasswd();
    }
    /**
     * Setter method for SQL query that is to be executed
     * @param sqlquery String representation of SQL query
     */
    public void setSqlQuery(String sqlquery){
	this.sqlStatement = sqlquery;
    }
    /**
     * Method returning parsed result set in form of abstract representation
     * @return
     */
    public ArrayList<HashMap<String, Object>> getList(){
	return this.resultList;
    }
}
