package com.example.groracle.Brokers;

import java.util.ArrayList;

import com.example.groracle.DAL.DAO.RoleDAO;
import com.example.groracle.DAL.DAO.UserDAO;
import com.example.groracle.Entity.Role;
import com.example.groracle.Entity.User;
/**
 * Class for operations with groups - providing abstraction of methods on group objects
 * @author tdosek
 * @version 1.0
 */
public class CommonGroupsBroker {
/**
 * Constructor for the class
 */
public CommonGroupsBroker(){
}
/**
 * Method for changing group belonging of given role
 * @param roleName Name of the role that needs group change
 * @param newGroups New groups granted to the role
 */
public void changeGroupsToRole(String roleName,ArrayList<String> newGroups){
    String groups = ";";
    for (String i: newGroups){
	groups = groups + i + ";";
    }
    RoleDAO rDal = new RoleDAO();
    Role role = rDal.findRole(roleName);
    role.setGroups(groups);
    rDal.updateRole(role);
    rDal.closeSession();
}
/**
 * Method for changing group belonging of a user
 * @param userName Name of the user that needs group change
 * @param newGroups New groups granted to the user
 */
public void changeGroupsToUser(String userName,ArrayList<String> newGroups){
    String groups = ";";
    for (String i: newGroups){
	groups = groups + i +";";
    }
    UserDAO uDal = new UserDAO();
    User user = uDal.findUser(userName);
    user.setGroups(groups);
    System.out.println("User is: " + user.getUsername());
    uDal.updateUser(user);
    uDal.closeSession();    
}
/**
 * Method providing validation of role belonging to group
 * @param groupname Name of the group
 * @param rolename Name of the role
 * @return Boolean value representing whether the role belongs to group (true) or not (false)
 */
public Boolean validateRoleGroupR(String groupname,String rolename){
    Boolean match = false;
    RoleDAO rDal = new RoleDAO();
    Role role = rDal.findRole(rolename);
    if(role != null){
	String groups = role.getGroups();
	if (groups.contains(groupname)) match=true;
	else match = false;
    }else match = false;
    rDal.closeSession();
    return match;
}
/**
 * Method providing validation of user belonging to a group
 * @param groupname Name of the group
 * @param username Name of the user
 * @return Boolean value representing whether the user belongs to group (true) or not (false)
 */
public Boolean validateUserGroupR(String groupname,String username){
    Boolean match = false;
    UserDAO uDal = new UserDAO();
    User user = uDal.findUser(username);
    if(user != null){
	String groups = user.getGroups();
	if (groups.contains(groupname)) match=true;
	else match = false;
    }else match = false;
    uDal.closeSession();
    return match;
}
}
