package com.example.groracle.Brokers;

import java.util.ArrayList;
import java.util.List;

import com.example.groracle.DAL.DAO.RightDAO;
import com.example.groracle.Entity.Right;
/**
 * Class providing base for permission operations
 * @author tdosek
 * @version 1.0
 */
public class CommonPermissionsBroker {
    /**
     * Constructor of the class
     */
    public CommonPermissionsBroker(){
    }
    /**
     * Method for changing permissions to a role
     * @param roleName Name of the role
     * @param newPermissions New permissions of the role
     */
    public void changePermissionToRole(String roleName, ArrayList<String> newPermissions){
	RightDAO rDal = new RightDAO();
	List<Right> actualPermsA = rDal.getRightsByRole(roleName);
	ArrayList<String> actualPerms = new ArrayList<>();
	for (int i=0; i<actualPermsA.size();i++){
	    actualPerms.add(actualPermsA.get(i).getPermission());
	}
	ArrayList<String> changesNegative = new ArrayList<>();
	for (int i=0; i<actualPerms.size();i++){
	    if (!newPermissions.contains(actualPerms.get(i))){
		changesNegative.add(actualPerms.get(i));
	    }
	}
	ArrayList<String> changesPositive = new ArrayList<>();
	for (int i=0; i<newPermissions.size();i++){
	    if(actualPerms.size()>0){
		if (!actualPerms.contains(newPermissions.get(i))){
		    changesPositive.add(newPermissions.get(i));
		}
	    }else changesPositive = newPermissions;
	}
	for (int i=0; i<changesPositive.size();i++){
	    Right newRight = rDal.findRight(changesPositive.get(i));
	    newRight.setRoles(newRight.getRoles() + roleName +";");
	    rDal.updateRight(newRight);
	}
	for (int i=0;i<changesNegative.size();i++){
	    Right obsoleteRight = rDal.findRight(changesNegative.get(i));
	    String obsoletePermissions = obsoleteRight.getRoles();
	    String updatedRoleString = obsoletePermissions.replace(";"+roleName+";", ";");
	    obsoleteRight.setRoles(updatedRoleString);
	    rDal.updateRight(obsoleteRight);
	}
	rDal.closeSession();
    }
    /**
     * Method for changing permissions of a user
     * @param userName Name of the user
     * @param newPermissions New permissions granted to a user
     */
    public void changePermissionToUser(String userName, ArrayList<String> newPermissions){
	RightDAO rDal = new RightDAO();
	List<Right> actualPermsA = rDal.getRightsByUser(userName);
	ArrayList<String> actualPerms = new ArrayList<>();
	for (int i=0; i<actualPermsA.size();i++){
	    actualPerms.add(actualPermsA.get(i).getPermission());
	}
	ArrayList<String> changesNegative = new ArrayList<>();
	for (int i=0; i<actualPerms.size();i++){
	    if (!newPermissions.contains(actualPerms.get(i))){
		changesNegative.add(actualPerms.get(i));
	    }
	}
	ArrayList<String> changesPositive = new ArrayList<>();
	for (int i=0; i<newPermissions.size();i++){
	    if(actualPerms.size()>0){
		if (!actualPerms.contains(newPermissions.get(i))){
		    changesPositive.add(newPermissions.get(i));
		}
	    }else changesPositive = newPermissions;
	}
	for (int i=0; i<changesPositive.size();i++){
	    Right newRight = rDal.findRight(changesPositive.get(i));
	    newRight.setUsernames(newRight.getUsernames() + userName +";");
	    rDal.updateRight(newRight);
	}
	for (int i=0;i<changesNegative.size();i++){
	    Right obsoleteRight = rDal.findRight(changesNegative.get(i));
	    String obsoletePermissions = obsoleteRight.getUsernames();
	    String updatedUnameString = obsoletePermissions.replace(";"+userName+";", ";");
	    obsoleteRight.setUsernames(updatedUnameString);
	    rDal.updateRight(obsoleteRight);
	}
	rDal.closeSession();
    }
    /**
     * Method providing validation functionality for users
     * @param permission Permission to be validated
     * @param username Name of the user whose permissions are validated
     * @return Boolean representation of the validation. True in case of valid user permissions, false on the opposite case
     */
    public Boolean validateUserPermission(String permission, String username){
	Boolean match = false;
	RightDAO rDal = new RightDAO();
	Right validatedRight = rDal.findRight(permission);
	if (validatedRight != null){
	    if (validatedRight.getUsernames().contains(username)) match = true;
	    else match = false;
	}else match = false;
	rDal.closeSession();
	return match;
    }
    /**
     * Method providing validation functionality for roles
     * @param permission Permission to be validated
     * @param rolename Name of the role
     * @return Boolean representation of the validation. True in case of valid user permissions, false on the opposite case
     */
    public Boolean validateRolePermission(String permission, String rolename){
	Boolean match = false;
	RightDAO rDal = new RightDAO();
	Right validatedRight = rDal.findRight(permission);
	if (validatedRight != null){
	    if (validatedRight.getRoles().contains(rolename)) match = true;
	    else match = false;
	}else match = false;
	rDal.closeSession();
	return match;
    }
}
