package com.example.groracle.Brokers;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.example.groracle.DAL.DAO.GrokLineDAO;
import com.example.groracle.DAL.DAO.PatternBlacklistDAO;
import com.example.groracle.Entity.PatternBlacklist;
import com.example.groracle.Entity.Agregated.GrokLine;
/**
 * Class responsible for generating the files with rules for grok
 * @author tdosek
 * @version 1.0
 */
public class GrokFileGenerator {
    private Boolean blacklist;
    private String optimizedGrokFile = null;
    /**
     * Constructor of the class
     * @param blacklist Boolean representation of whether file should be blacklist, or not.
     */
    public GrokFileGenerator(Boolean blacklist){
	this.blacklist = blacklist;
    }
    /**
     * Method responsible for generating the file content
     */
    public void generateFileString(){
	ConfigLoader confLoad = new ConfigLoader();
	confLoad.loadConfig();
	String grokFileInput = confLoad.getGrokRulesFile();
	StringBuilder grokFileBuilder = new StringBuilder();
	GrokLineDAO glDao = new GrokLineDAO();
	PatternBlacklistDAO pDao = new PatternBlacklistDAO();
	if (blacklist == false){
	    try {
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(grokFileInput)));
		while(br.ready()){
		    grokFileBuilder.append(br.readLine()+"\n");
		}
	    } catch (FileNotFoundException e) {
		System.out.println("Failed to read from the grok input file. Is it present and readable?");
		e.printStackTrace();
	    } catch (IOException e){
		System.out.println("Failed to read from the grok input file. Is it present and readable?");
		e.printStackTrace();
	    }
	    List<GrokLine> dbPatternResult = glDao.getAllLines(true);
	    if (dbPatternResult.size()>0){
		for (GrokLine i: dbPatternResult){
		    String pattern = i.getPattern();
		    PatternBlacklist blacklistedPattern= pDao.findPatternBlackList(pattern);
		    String name = blacklistedPattern.getName();
		    String finalPattern = null;
		    if (name == null || name.equals("")){
			finalPattern = "pattern"+ i.getCount() + " " + pattern + "\n";
		    }else finalPattern = name + " " + pattern + "\n";
		    if(!grokFileBuilder.toString().contains(finalPattern))grokFileBuilder.append(finalPattern);
	    	}
	    }
	}else{
	    List<GrokLine> dbPatternResult = glDao.getAllLines(false);
	    if (dbPatternResult.size()>0){
		for (GrokLine i: dbPatternResult){
		    String pattern = i.getPattern();
		    PatternBlacklist blacklistedPattern= pDao.findPatternBlackList(pattern);
		    String name = blacklistedPattern.getName();
		    String finalPattern = null;
		    if (name == null || name.equals("")){
			finalPattern = "pattern"+ i.getCount() + " " + pattern + "\n";
		    }else finalPattern = name + " " + pattern + "\n";
		    if(!grokFileBuilder.toString().contains(finalPattern))grokFileBuilder.append(finalPattern);
	    	}
	    }
	}
	pDao.closeSession();
	glDao.closeSession();
	optimizedGrokFile = grokFileBuilder.toString();
    }
    /**
     * Getter method returning file content for optimized rules file
     * @return file content
     */
    public String getOptimizedGrokFile(){
	return this.optimizedGrokFile;
    }

}
