package com.example.groracle.DAL.DAO;

import java.util.List;

import org.hibernate.SQLQuery;
import org.hibernate.Session;

import com.example.groracle.DAL.SessionFactoryUtil;
import com.example.groracle.Entity.PatternBlacklist;
/**
 * Class providing operations on top of PatternBlacklist object
 * @author tdosek
 * @version 1.1
 */
public class PatternBlacklistDAO {
    private Session ss = null;
    /**
     * Constructor of the class - only thing to do here is to open session to DB
     */
    public PatternBlacklistDAO(){
	ss = SessionFactoryUtil.getSessionFactory().openSession();
    }
    /**
     * Method returning list of all pattern_blacklist entries
     * @return list of pattern_blacklist entries 
     */
    public List<PatternBlacklist> getAllPatterns(){
	ss.beginTransaction();
	List<PatternBlacklist> result = ss.createQuery( "from PatternBlacklist" ).list();
	ss.getTransaction().commit();
	return result;
    }
    /**
     * Method returning list of confirmed or unconfirmed blacklisted patterns
     * @param confirmed Parameter that defines whether we should return confirmed entries or unconfirmed ones
     * @return List of entries matching the criteria
     */
    public List<PatternBlacklist> getConfirmedOrBlacklisted(Boolean confirmed){
	ss.beginTransaction();
	String sQuery = "SELECT * FROM pattern_blacklist WHERE confirmed = '"+confirmed+"'";
	SQLQuery query = ss.createSQLQuery(sQuery).addEntity(PatternBlacklist.class);
	List<PatternBlacklist> result = query.list();
	ss.getTransaction().commit();
	ss.disableFilter("pattern_filter");
	return result;
    }
    /**
     * Method for finding a blacklist entry based on pattern name
     * @param pattern pattern string of the searched item
     * @return Blacklist item in case it exists, otherwise null
     */
    public PatternBlacklist findPatternBlackList(String pattern){
	ss.beginTransaction();
	PatternBlacklist resPatternBlacklist = (PatternBlacklist) ss.get(PatternBlacklist.class, pattern);
	ss.getTransaction().commit();
	if (resPatternBlacklist != null){
	    return resPatternBlacklist;
	}else{
	    return null;
	}
    }
    /**
     * Method providing functionality for creating new pattern blacklist
     * @param patternBlacklistItem Item to be created
     * @return Return true in case the creation succeeds, false otherwise
     */
    public Boolean newPatternBlacklist(PatternBlacklist patternBlacklistItem){
	if (findPatternBlackList(patternBlacklistItem.getPattern()) == null){
	    ss.beginTransaction();
	    ss.save(patternBlacklistItem);
	    ss.getTransaction().commit();
	    return true;
	}else{
	    return false;
	}	
    }
    /**
     * Method providing functionality for updating pattern blacklist item
     * @param patternBlacklistItem item to be updated
     */
    public void updatePatternBlacklist(PatternBlacklist patternBlacklistItem){
	ss.beginTransaction();
	ss.merge(patternBlacklistItem);
	ss.getTransaction().commit();
    }
    /**
     * Method providing functionality for deleting a pattern blacklist item
     * @param patternBlacklistItem Item to be deleted
     */
    public void deleteBlackList(PatternBlacklist patternBlacklistItem){
	ss.beginTransaction();
	ss.delete(patternBlacklistItem);
	ss.getTransaction().commit();
    }
    /**
     * Method for closing the session - it flushes database connection
     */
    public void closeSession(){
	ss.flush();
	ss.clear();
	ss.close();
    }
}
