package com.example.groracle.Views;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.GroupDAO;
import com.example.groracle.Entity.Group;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class AddGroupView represents UI handler for adding new groups into the system.
 * Group is base for providing functionality like mass data upload.
 * @author tdosek
 * @version 1.0
 */
public class AddGroupView extends Panel implements View{
    public static final String NAME = "AddGroup";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    /**
     * Constructor for class AddGroupView.
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public AddGroupView(Navigator navigator){
	this.navigator = navigator;
    }    
    /**
     * Initialization method which handles composition of UI for given class
     */
    protected void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    /**
     * Initialization method for preparing layout of editor for given functionality 
     */
    private void initEditor(){
	VerticalLayout editor = new VerticalLayout();
	editor.setStyleName("loginEditor");
	header.addComponent(editor);
	
	Label groupL = new Label("Group name");
	editor.addComponent(groupL);
	final TextField groupF = new TextField("");
	editor.addComponent(groupF);
	groupL.setStyleName("loginName");
	groupF.setStyleName("loginUserText");
	
	Label descL = new Label("Description");
	editor.addComponent(descL);
	final TextArea descF = new TextArea();
	editor.addComponent(descF);
	descL.setStyleName("loginName");
	descF.setStyleName("loginUserText");
	
	Button createGroupB = new Button("Create group");
	createGroupB.setStyleName("loginButton");
	editor.addComponent(createGroupB);
	
	createGroupB.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		String groupName = groupF.getValue();
		if ((!groupName.equals(null)) && (!groupName.equals(""))){
		    if (!groupName.contains(";")){
			Group newGroup = new Group();
			newGroup.setGroupid(groupName);
			newGroup.setDescription(descF.getValue());
			GroupDAO groupDal = new GroupDAO();
			if(groupDal.newGroup(newGroup)){
			    init();
			    Notification.show("Group succesfully created");
			}else Notification.show("Failed to create group","Group name already exists",Notification.TYPE_ERROR_MESSAGE);
			groupDal.closeSession();
		    }else Notification.show("Failed to create group","Group name can't contain ; sign",Notification.TYPE_ERROR_MESSAGE);
		}else Notification.show("Failed to create group","Group name can't be empty",Notification.TYPE_ERROR_MESSAGE);
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();
    }

}
