package com.example.groracle.Views;

import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.InputDAO;
import com.example.groracle.DAL.DAO.PatternBlacklistDAO;
import com.example.groracle.Entity.Input;
import com.example.groracle.Entity.PatternBlacklist;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of view for confirmed patterns and relevant operations
 * @author tdosek
 * @version 1.0
 */
public class ConfirmedView extends Panel implements View{
    public static final String NAME = "ConfirmedList";
    private Navigator navigator = null;
    // Initializing variables for view
    private VerticalLayout header = new VerticalLayout();
    // Initializing variables for blacklist view
    private IndexedContainer blacklist = createBlacklistDatasource();
    private Button deleteFromBlacklistButton = new Button("Remove pattern from list of confirmed ones");
    private FormLayout blacklistEditorLayout = new FormLayout();
    private static String blacklistedPattern = "Confirmed pattern";
    private static String blacklistedPatternName = "Name of pattern";
    private static final String[] blacklistFieldNames = new String[] {blacklistedPatternName,blacklistedPattern};
    private Table blackList = new Table();
    // Inititalizing variables for details of given pattern
    private IndexedContainer blackListDetailsList = new IndexedContainer();
    private Table blackListDetails = new Table();
    private static String datePattern = "Date";
    private static String hostPattern = "Host";
    private static String linePattern = "Line";
    private static final String[] blackListDetailsFieldNames = new String[]{datePattern,hostPattern,linePattern};
    // 	Variables for storing pattern selected in UI
    private String selectedPattern = null;
    private Object lineId = null;
    /**
     * Cosntructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public ConfirmedView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	selectedPattern = null;
	lineId = null;
	blacklist = createBlacklistDatasource();
	initBlackList();
	initBlacklistLayout();
	initBlackListEditor();
    }
    @SuppressWarnings("deprecation")
    /**
     * Initialization method for UI layout of the view
     */
    private void initBlacklistLayout(){
	blackList.setStyleName("v-table-strong");
	blackListDetails.setStyleName("v-table-strong");
	final VerticalLayout leftLayout = new VerticalLayout();
	header.addComponent(leftLayout);	    
	header.addComponent(blacklistEditorLayout);
	    
	leftLayout.addComponent(blackList);
	blackList.setHeight("100%");
	leftLayout.setSizeFull();
	leftLayout.setExpandRatio(blackList, 1);
	
	blackList.addListener(new Property.ValueChangeListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		blacklistEditorLayout.removeComponent(blackListDetails);
		lineId = blackList.getValue();
		if (lineId != null){
		    blackListDetailsList = createBlacklistDetailsDatasource(blackList.getItem(lineId)
			    .getItemProperty(blacklistedPattern).getValue().toString());
		    blackListDetails.setContainerDataSource(blackListDetailsList);
		    blackListDetails.setVisibleColumns(blackListDetailsFieldNames);
		    blackListDetails.setSelectable(true);
		    blackListDetails.setImmediate(true);
		    blackListDetails.setWidth("100%");
		    blacklistEditorLayout.addComponent(blackListDetails);
		}
	    }
	});
    }
    /**
     * Initialization method for editor layout
     */
    private void initBlackListEditor(){
	blackList.setWidth("100%");
	if (blacklistEditorLayout.getComponentCount() == 0){
	    blacklistEditorLayout.addComponent(deleteFromBlacklistButton);
	    deleteFromBlacklistButton.addClickListener(new ClickListener() {
		    
	    @Override
	    public void buttonClick(ClickEvent event) {
		Object line = blackList.getValue();
		if (line != null){
		    PatternBlacklistDAO pDal = new PatternBlacklistDAO();
		    PatternBlacklist delPattern = pDal.findPatternBlackList(selectedPattern);
		    pDal.deleteBlackList(delPattern);
		    pDal.closeSession();
		    blackList.removeItem(lineId);
		    blacklist.removeItem(lineId);
		    blackList.refreshRowCache();
		    Notification.show("Pattern successfully deleted from blacklist");
		}
	    }
	});
    	deleteFromBlacklistButton.setStyleName("loginButton");	   
    	}
    }
    /**
     * Initialization method for preparing the main table of the view
     */
    private void initBlackList(){
	blackList.setContainerDataSource(blacklist);
	blackList.setVisibleColumns(blacklistFieldNames);
	blackList.setSelectable(true);
	blackList.setImmediate(true);
	blackList.addValueChangeListener(new Property.ValueChangeListener() {
	        
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		lineId = blackList.getValue();
	    	if (lineId != null){
	    	    selectedPattern = blackList.getItem(lineId)
			    .getItemProperty(blacklistedPattern).getValue().toString();
	 	}	
	    }
	});
	blackList.setBuffered(false);
    }
    /**
     * Initializing Data Sources for Tables in UI
     */
    private static IndexedContainer createBlacklistDatasource(){
    // 	Loading database values into an Indexed container for further usage by UI.
	IndexedContainer ic = new IndexedContainer();
	for (String p : blacklistFieldNames){
	    ic.addContainerProperty(p, String.class, "");
	}
	PatternBlacklistDAO pDal = new PatternBlacklistDAO();
	List<PatternBlacklist> blacklist = pDal.getConfirmedOrBlacklisted(true);
	for (int i = 0; i < blacklist.size(); i++){
	    Object id = ic.addItem();
	    String blacklistName = blacklist.get(i).getName();
	    String blacklistAdd = blacklist.get(i).getPattern();
	    ic.getContainerProperty(id, blacklistedPatternName).setValue(blacklistName);
	    ic.getContainerProperty(id, blacklistedPattern).setValue(blacklistAdd);
	}
	pDal.closeSession();
	return ic;
    }
    /**
     * Method for initialization of details table content
     * @param pattern Pattern whose details are to be displayed
     * @return variable representing details of the selected item
     */  
    private IndexedContainer createBlacklistDetailsDatasource(String pattern){
	IndexedContainer ic = new IndexedContainer();
	for (String p: blackListDetailsFieldNames){
	    ic.addContainerProperty(p, String.class, "");
	}
	InputDAO inDal = new InputDAO();
	List<Input> details = inDal.getFilteredInput(selectedPattern);

	for (int i = 0; i < details.size(); i++){
	    Object id = ic.addItem();
	    Input row = details.get(i);
	    String date = row.getDate();
	    String host = row.getHost();
	    String line = row.getLine();
	    ic.getContainerProperty(id, datePattern).setValue(date);
	    ic.getContainerProperty(id, hostPattern).setValue(host);
	    ic.getContainerProperty(id, linePattern).setValue(line);
	}
	inDal.closeSession();
	return ic;
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }    
}
