package com.example.groracle.Views;

import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.GroupDAO;
import com.example.groracle.Entity.Group;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of operations for editting groups
 * @author tdosek
 * @version 1.0
 */
public class EditGroupView extends Panel implements View{
    public static final String NAME = "EditGroup";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    // Initializing variables for the editor
    private IndexedContainer grouplist = null;
    private Table groupList = new Table();
    private static String groupName = "Group name";
    private static String groupDescription = "Group description";
    private FormLayout grouplistEditorLayout = new FormLayout();
    private FieldGroup editorFields = new FieldGroup();
    private final TextField groupF = new TextField(groupName);
    private final TextArea descA = new TextArea(groupDescription);
    // Initializing variables for selected items
    private String selectedGroup = null;
    private Object lineId = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public EditGroupView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	grouplist = createGroupListDatasource();
	initListLayout();
	initEditor();
	initList();
    }
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	groupList.setStyleName("v-table-strong");
	VerticalLayout editor = new VerticalLayout();
	header.addComponent(editor);
	if (grouplistEditorLayout.getComponentCount()==0){
	    grouplistEditorLayout.addComponent(groupF);
	    grouplistEditorLayout.addComponent(descA);
	    editorFields.bind(groupF, groupName);
	    editorFields.bind(descA, groupDescription);
	    
	    Button editGroupB = new Button("Save change");
	    grouplistEditorLayout.addComponent(editGroupB);
	    
	    editGroupB.addClickListener(new ClickListener() {
	        
	        @Override
	        /*
	         * (non-Javadoc)
	         * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	         */
	        public void buttonClick(ClickEvent event) {
		    String groupname = groupF.getValue();
		    if (!groupname.contains(";")){
			if ((!groupname.equals(null)) && (!groupname.equals(""))){
			    GroupDAO gDal = new GroupDAO();
			    Group changedGroup = gDal.findGroup(selectedGroup); 
			    changedGroup.setDescription(descA.getValue());
			    gDal.updateGroup(changedGroup);
			    if (!selectedGroup.equals(groupname))gDal.updateGroupName(selectedGroup, groupname);
			    groupF.setValue("");
			    descA.setValue("");
			    Notification.show("Group succesfully updated");
			    gDal.closeSession();
			    init();
			}else Notification.show("Failed to update group","Group name can't be empty",Notification.TYPE_ERROR_MESSAGE);
		    }else Notification.show("Failed to update Group","Group can't contain ; sign",Notification.TYPE_ERROR_MESSAGE);
	        }
	    });
	    
	    Button deleteRoleB = new Button("Delete group");
	    grouplistEditorLayout.addComponent(deleteRoleB);
	    deleteRoleB.addClickListener(new ClickListener() {
	        
	        @Override
	        /*
	         * (non-Javadoc)
	         * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	         */
	        public void buttonClick(ClickEvent event) {
	            if (!(selectedGroup == null)){
	        	GroupDAO gDal = new GroupDAO();
	        	Group deletedGroup = gDal.findGroup(selectedGroup);
	        	gDal.deleteGroup(deletedGroup);
	        	gDal.closeSession();
	        	groupF.setValue("");
	        	descA.setValue("");
	        	selectedGroup = null;
			lineId = null;
			init();  
	            }else Notification.show("Failed to delete group","You have to select a group first",Notification.TYPE_ERROR_MESSAGE);	    	
	        }
	    }); 
	}
    }
    /**
     * Method for creating datasource of the role list
     * @return role data
     */
    private static IndexedContainer createGroupListDatasource() {
	IndexedContainer ic = new IndexedContainer();
	for (String p : new String[]{groupName,groupDescription}){
	    ic.addContainerProperty(p, String.class, "");
	}
	GroupDAO gDal = new GroupDAO();
	List<Group> allGroups = gDal.getAllGroups();
	for (Group i: allGroups){
	    Object id = ic.addItem();
	    String groupNameAdd = i.getGroupid();
	    String groupDescAdd = i.getDescription();
	    ic.getContainerProperty(id, groupName).setValue(groupNameAdd);
	    ic.getContainerProperty(id, groupDescription).setValue(groupDescAdd);
	}
	gDal.closeSession();
	return ic;
    }
    /**
     * Method initializing layout of relevant table
     */
    private void initListLayout(){
	final VerticalLayout leftLayout = new VerticalLayout();
	header.addComponent(leftLayout);	    
	header.addComponent(grouplistEditorLayout);
	    
	leftLayout.addComponent(groupList);
	groupList.setHeight("100%");
	groupList.setWidth("100%");
	leftLayout.setSizeFull();
	leftLayout.setExpandRatio(groupList, 1);
    }
    /**
     * Method initializing Table and mapping relevant objects to it
     */
    private void initList(){
	groupList.setContainerDataSource(grouplist);
	groupList.setVisibleColumns(new String[]{groupName, groupDescription});
	groupList.setSelectable(true);
	groupList.setImmediate(true);
	groupList.addValueChangeListener(new Property.ValueChangeListener() {
	        
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		lineId = groupList.getValue();
	    	if (lineId != null){
	    	    editorFields.setItemDataSource(groupList.getItem(lineId));
	    	    selectedGroup = groupList.getItem(lineId).getItemProperty(groupName).getValue().toString();
	 	}	
	    }
	});
	groupList.setBuffered(false);
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
