package com.example.groracle.Views;

import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.GroupDAO;
import com.example.groracle.DAL.DAO.HostDAO;
import com.example.groracle.Entity.Group;
import com.example.groracle.Entity.Host;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.HorizontalSplitPanel;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of editing group belonging of hosts
 * @author tdosek
 * @version 1.0
 */
public class EditHostGroupView extends Panel implements View{
    public static final String NAME = "EditHostGroups";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    private final OptionGroup hosts = new OptionGroup();
    private final OptionGroup groups = new OptionGroup();
    private String selectedHost = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public EditHostGroupView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    @SuppressWarnings("deprecation")
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	header.setSizeFull();
	HorizontalLayout editor = new HorizontalLayout();
	editor.setWidth("100%");
	header.addComponent(editor);
	
	HorizontalSplitPanel hsplit = new HorizontalSplitPanel();
	hsplit.setHeight("100%");
	hsplit.setWidth("100%");
	editor.addComponent(hsplit);
	VerticalLayout leftLayout = new VerticalLayout();
	VerticalLayout rightLayout = new VerticalLayout();
	
	hsplit.setFirstComponent(leftLayout);
	hsplit.setSecondComponent(rightLayout);
	
	Label hostsLabel = new Label("Hosts");
	Label groupsLabel = new Label("Groups");
	leftLayout.addComponent(hostsLabel);
	leftLayout.addComponent(hosts);
	rightLayout.addComponent(groupsLabel);
	rightLayout.addComponent(groups);
	hostsLabel.setStyleName("loginName");
	groupsLabel.setStyleName("loginName");

	final HostDAO hDal = new HostDAO();
	List<Host> hostArray = hDal.getAllHosts();
	for (Host i: hostArray){
	   hosts.addItem(i.getHost());
	}
	final GroupDAO gDal = new GroupDAO();
	List<Group> groupsArray = gDal.getAllGroups();
	if (!(groupsArray.size()==0)){
	    for (Group i: groupsArray){
		groups.addItem(i.getGroupid());
	    }
	    groups.addItem("None");
	}
	hDal.closeSession();
	gDal.closeSession();
	groups.setStyleName("v-select-optiongroup-horizontalgroup");
	groups.setImmediate(true);
	hosts.setImmediate(true);
	hosts.setStyleName("v-select-optiongroup-horizontalgroup");
	hosts.addListener( new Property.ValueChangeListener() {

	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		selectedHost = hosts.getValue().toString();
		HostDAO h1Dal = new HostDAO();
		for (Object item: groups.getItemIds()){
		    groups.unselect(item);
		}
		Host selection = h1Dal.findHost(selectedHost);
		if (selection != null){
		    String group = selection.getGroupid();
		    if (!(group==null) && !(group.equals("*")) && !(group.equals("")))groups.select(group);
		    else groups.select("None");
		}
		h1Dal.closeSession();
	    }
	});
	
	Button submitB = new Button("Submit changes");
	header.addComponent(submitB);
	submitB.addClickListener(new Button.ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		HostDAO h1Dal = new HostDAO();
		if(!(selectedHost.equals("")) && !(selectedHost==null)){
		    if(groups.getValue().equals("None")){
			Host updatedHost = h1Dal.findHost(selectedHost);
			updatedHost.setGroupid("");
			h1Dal.updateHost(updatedHost);
		    }else{
			Host updatedHost = h1Dal.findHost(selectedHost);
			updatedHost.setGroupid(groups.getValue().toString());
			h1Dal.updateHost(updatedHost);
		    }
		    Notification.show("Host groups successfully updated");
		    h1Dal.closeSession();
		    init();
		}else Notification.show("Failed to update host groups","You have to select a host first",Notification.TYPE_ERROR_MESSAGE);
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
