package com.example.groracle.Views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.CommonGroupsBroker;
import com.example.groracle.DAL.DAO.GroupDAO;
import com.example.groracle.DAL.DAO.RoleDAO;
import com.example.groracle.Entity.Group;
import com.example.groracle.Entity.Role;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.HorizontalSplitPanel;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of editing role group belonging
 * @author tdosek
 * @version 1.0
 */
public class EditRoleGroupsView extends Panel implements View{
    public static final String NAME = "EditRoleGroups";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    private final OptionGroup roles = new OptionGroup();
    private final OptionGroup groups = new OptionGroup();
    private String selectedRole = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public EditRoleGroupsView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    @SuppressWarnings("deprecation")
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	header.setSizeFull();
	HorizontalLayout editor = new HorizontalLayout();
	editor.setWidth("100%");
	header.addComponent(editor);
	
	groups.setMultiSelect(true);
	
	HorizontalSplitPanel hsplit = new HorizontalSplitPanel();
	hsplit.setHeight("100%");
	hsplit.setWidth("100%");
	editor.addComponent(hsplit);
	VerticalLayout leftLayout = new VerticalLayout();
	VerticalLayout rightLayout = new VerticalLayout();
	
	hsplit.setFirstComponent(leftLayout);
	hsplit.setSecondComponent(rightLayout);
	
	Label rolesLabel = new Label("Roles");
	Label groupsLabel = new Label("Groups");
	leftLayout.addComponent(rolesLabel);
	leftLayout.addComponent(roles);
	rightLayout.addComponent(groupsLabel);
	rightLayout.addComponent(groups);
	rolesLabel.setStyleName("loginName");
	groupsLabel.setStyleName("loginName");
	
	RoleDAO rDal = new RoleDAO();
	List<Role> allRoles = rDal.getAllRoles();
	for (Role i: allRoles){
	    roles.addItem(i.getRole());
	}
	rDal.closeSession();
	
	GroupDAO gDal = new GroupDAO();
	List<Group> allGroups = gDal.getAllGroups();
	for (Group i: allGroups){
		groups.addItem(i.getGroupid());
	}
	gDal.closeSession();
	groups.setStyleName("v-select-optiongroup-horizontalgroup");
	groups.setImmediate(true);
	roles.setImmediate(true);
	roles.addListener( new Property.ValueChangeListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		// first we need to unselect all
		selectedRole = roles.getValue().toString();
		for (Object item: groups.getItemIds()){
		    if (selectedRole.equals("admin")) groups.select(item);
		    else groups.unselect(item);
		}
		RoleDAO rDal = new RoleDAO();
		Role actualRole = rDal.findRole(selectedRole);
		if (actualRole != null){
		    String groupList = actualRole.getGroups();
		    String[] list = groupList.split("\\;");
		    for (String i: list){
			if (!(i==null) && !(i.equals("*")) && !(i.equals("")))groups.select(i);
		    }
		}
		rDal.closeSession();
	    }
	});
	Button submitB = new Button("Submit changes");
	header.addComponent(submitB);
	submitB.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		if (!(selectedRole == null)){
		    if (!selectedRole.equals("admin")){
			ArrayList<String> selection = new ArrayList<>();
			for (Object i: (Collection)groups.getValue()){
			    selection.add(i.toString());
			}				
			CommonGroupsBroker gb = new CommonGroupsBroker();
			gb.changeGroupsToRole(selectedRole, selection);
			Notification.show("Role groups successfully updated");
			init();
		    }else Notification.show("Failed to update role groups","Admin role groups can't be editted",Notification.TYPE_ERROR_MESSAGE);
		}else Notification.show("Failed to upgrade role groups","You have to select a role first",Notification.TYPE_ERROR_MESSAGE);
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
