package com.example.groracle.Views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.CommonPermissionsBroker;
import com.example.groracle.DAL.DAO.RightDAO;
import com.example.groracle.DAL.DAO.RoleDAO;
import com.example.groracle.Entity.Right;
import com.example.groracle.Entity.Role;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.HorizontalSplitPanel;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of editing roles' permissions
 * @author tdosek
 * @version 1.0
 */
public class EditRolePermissionsView extends Panel implements View{
    public static final String NAME = "EditRolePermissions";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    private final OptionGroup roles = new OptionGroup();
    private final OptionGroup permissions = new OptionGroup();
    private String selectedRole = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public EditRolePermissionsView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    protected void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    @SuppressWarnings("deprecation")
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	header.setSizeFull();
	HorizontalLayout editor = new HorizontalLayout();
	editor.setWidth("100%");
	header.addComponent(editor);
	
	permissions.setMultiSelect(true);
	
	HorizontalSplitPanel hsplit = new HorizontalSplitPanel();
	hsplit.setHeight("100%");
	hsplit.setWidth("100%");
	editor.addComponent(hsplit);
	VerticalLayout leftLayout = new VerticalLayout();
	VerticalLayout rightLayout = new VerticalLayout();
	
	hsplit.setFirstComponent(leftLayout);
	hsplit.setSecondComponent(rightLayout);
	
	Label rolesLabel = new Label("Roles");
	Label permissionsLabel = new Label("Permissions");
	leftLayout.addComponent(rolesLabel);
	leftLayout.addComponent(roles);
	rightLayout.addComponent(permissionsLabel);
	rightLayout.addComponent(permissions);
	rolesLabel.setStyleName("loginName");
	permissionsLabel.setStyleName("loginName");
	
	RoleDAO rDal = new RoleDAO();
	List<Role> allRoles = rDal.getAllRoles();
	for (Role i: allRoles){
	    roles.addItem(i.getRole());
	}
	rDal.closeSession();
	
	RightDAO riDal = new RightDAO();
	List<Right> allRights = riDal.getAllRights();
	for (Right i: allRights){
	    permissions.addItem(i.getPermission());
	}
	riDal.closeSession();
	permissions.setStyleName("v-select-optiongroup-horizontalgroup");
	permissions.setImmediate(true);
	roles.setImmediate(true);
	roles.addListener( new Property.ValueChangeListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		// first we need to unselect all
		selectedRole = roles.getValue().toString();
		for (Object item: permissions.getItemIds()){
		    permissions.unselect(item);
		}
		RightDAO riDal = new RightDAO();
		List<Right> roleRights = riDal.getRightsByRole(selectedRole);
		for (Right i: roleRights){
		    permissions.select(i.getPermission());
		}
		riDal.closeSession();
	    }
	});
	Button submitB = new Button("Submit changes");
	header.addComponent(submitB);
	submitB.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		if (!(selectedRole == null)){
		    if (!selectedRole.equals("admin")){
			ArrayList<String> selection = new ArrayList<>();
			for (Object i: (Collection)permissions.getValue()){
			    selection.add(i.toString());
			}				
			if (selection.size() != 0){
			    CommonPermissionsBroker pb = new CommonPermissionsBroker();
			    pb.changePermissionToRole(selectedRole, selection);
			    Notification.show("Role permissions successfully updated");
			    init();
			}else Notification.show("Failed to update role persmissions","At least one permission has to be granted",Notification.TYPE_ERROR_MESSAGE);		
		    }else Notification.show("Failed to update role permissions","Admin role permissions can't be editted",Notification.TYPE_ERROR_MESSAGE);
		}else Notification.show("Failed to upgrade role permissions","You have to select a role first",Notification.TYPE_ERROR_MESSAGE);
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
