package com.example.groracle.Views;

import java.util.ArrayList;
import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.CommonPermissionsBroker;
import com.example.groracle.DAL.DAO.RoleDAO;
import com.example.groracle.DAL.DAO.UserDAO;
import com.example.groracle.Entity.Role;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of role editing
 * @author tdosek
 * @version 1.0
 */
public class EditRoleView extends Panel implements View {
    public static final String NAME = "EditRole";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    // Initialize variables for the view
    private IndexedContainer rolelist = null;
    private Table roleList = new Table();
    private static String rolename = "Role name";
    private FormLayout rolelistEditorLayout = new FormLayout();
    private FieldGroup editorFields = new FieldGroup();
    private final TextField roleF = new TextField(rolename);
    // Initialization of variables for selecting items
    private String selectedRole = null;
    private Object lineId = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public EditRoleView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method for UI layout of the editor
     */
    protected void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	rolelist = createRoleListDatasource();
	initListLayout();
	initEditor();
	initList();
    }
    /**
     * Method for creating datasource of the role list
     * @return role data
     */
    private static IndexedContainer createRoleListDatasource() {
	IndexedContainer ic = new IndexedContainer();
	for (String p : new String[]{rolename}){
	    ic.addContainerProperty(p, String.class, "");
	}
	RoleDAO rDal = new RoleDAO();
	List<Role> allRoles = rDal.getAllRoles();
	for (Role i: allRoles){
	    Object id = ic.addItem();
	    String roleNameAdd = i.getRole();
	    ic.getContainerProperty(id, rolename).setValue(roleNameAdd);
	}
	rDal.closeSession();
	return ic;
    }
    /**
     * Method for intialization of table layout
     */
    private void initListLayout(){
	roleList.setStyleName("v-table-strong");
	final VerticalLayout leftLayout = new VerticalLayout();
	header.addComponent(leftLayout);	    
	header.addComponent(rolelistEditorLayout);
	    
	leftLayout.addComponent(roleList);
	roleList.setHeight("100%");
	roleList.setWidth("100%");
	leftLayout.setSizeFull();
	leftLayout.setExpandRatio(roleList, 1);
    }
    /**
     * Method for intialization of the editor layout 
     */
    protected void initEditor(){
	VerticalLayout editor = new VerticalLayout();
	editor.setStyleName("loginEditor");
	header.addComponent(editor);
	if (rolelistEditorLayout.getComponentCount() == 0){
	    rolelistEditorLayout.addComponent(roleF);
	    editorFields.bind(roleF, rolename);
	
	    Button editRoleB = new Button("Save change");
	    rolelistEditorLayout.addComponent(editRoleB);
	
	    editRoleB.addClickListener(new ClickListener() {
	    
		@Override
		/*
		 * (non-Javadoc)
		 * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
		 */
		public void buttonClick(ClickEvent event) {
		    String roleName = roleF.getValue();
		    if (!selectedRole.equals("admin")){
			if (!roleName.contains(";")){
			    if ((!roleName.equals(null)) && (!roleName.equals(""))){
				RoleDAO rDal = new RoleDAO();
				Role findRole = rDal.findRole(roleName);
				if (findRole == null){
				    rDal.updateRoleName(selectedRole,roleName);
				    UserDAO uDal = new UserDAO();
				    uDal.onRoleUpdate(selectedRole, roleName);
				    uDal.closeSession();
				    Notification.show("Role succesfully updated");
				    init();
				}else Notification.show("Failed to update role","Role name already exists",Notification.TYPE_ERROR_MESSAGE);
				rDal.closeSession();
			    }else Notification.show("Failed to update role","Role name can't be empty",Notification.TYPE_ERROR_MESSAGE);
			}else Notification.show("Failed to update role","Role can't contain ; sign",Notification.TYPE_ERROR_MESSAGE);
		    }else Notification.show("Failed to update role","Admin role cannot be changed",Notification.TYPE_ERROR_MESSAGE);
		}
	    });
	
	    Button deleteRoleB = new Button("Delete role");
	    rolelistEditorLayout.addComponent(deleteRoleB);
	    deleteRoleB.addClickListener(new ClickListener() {
	    
		@Override
		/*
		 * (non-Javadoc)
		 * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
		 */
		public void buttonClick(ClickEvent event) {
		    if (!(selectedRole.equals("admin") || selectedRole.equals("user"))){
			if (!(selectedRole == null)){
			    RoleDAO rDal = new RoleDAO();
			    Role deletedRole = rDal.findRole(selectedRole);
			    rDal.deleteRole(deletedRole);
			    rDal.closeSession();
			    UserDAO uDal = new UserDAO();
			    uDal.onRoleUpdate(selectedRole, null);
			    uDal.closeSession();
			    ArrayList<String> removeRoles = new ArrayList<>();
			    CommonPermissionsBroker pb = new CommonPermissionsBroker();
			    pb.changePermissionToRole(selectedRole, removeRoles);
			    Notification.show("Role successfully removed");
			    roleF.setValue("");
			    selectedRole = null;
			    init();
		    	}else Notification.show("Failed to remove role","You have to select a role first",Notification.TYPE_ERROR_MESSAGE);
		    }else Notification.show("Failed to remove role","Admin and user role can't be removed",Notification.TYPE_ERROR_MESSAGE);
		}
	    });
	}
    }
    /**
     * Method mapping table to the relevant data source
     */
    private void initList(){
	roleList.setContainerDataSource(rolelist);
	roleList.setVisibleColumns(new String[]{rolename});
	roleList.setSelectable(true);
	roleList.setImmediate(true);
	roleList.addValueChangeListener(new Property.ValueChangeListener() {
	        
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		lineId = roleList.getValue();
	    	if (lineId != null){
	    	    editorFields.setItemDataSource(roleList.getItem(lineId));
	    	    selectedRole = roleList.getItem(lineId).getItemProperty(rolename).getValue().toString();
	 	}	
	    }
	});
	roleList.setBuffered(false);
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
