package com.example.groracle.Views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.CommonPermissionsBroker;
import com.example.groracle.DAL.DAO.RightDAO;
import com.example.groracle.DAL.DAO.UserDAO;
import com.example.groracle.Entity.Right;
import com.example.groracle.Entity.User;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.HorizontalSplitPanel;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of editing users' permissions 
 * @author tdosek
 * @version 1.0
 */
public class EditUserPermissionsView extends Panel implements View{
    public static final String NAME = "EditUserPermissions";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    private final OptionGroup users = new OptionGroup();
    private final OptionGroup permissions = new OptionGroup();
    private String selectedUser = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching
     */
    public EditUserPermissionsView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    @SuppressWarnings("deprecation")
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	header.setSizeFull();
	HorizontalLayout editor = new HorizontalLayout();
	editor.setWidth("100%");
	header.addComponent(editor);
	
	permissions.setMultiSelect(true);
	
	HorizontalSplitPanel hsplit = new HorizontalSplitPanel();
	hsplit.setHeight("100%");
	hsplit.setWidth("100%");
	editor.addComponent(hsplit);
	VerticalLayout leftLayout = new VerticalLayout();
	VerticalLayout rightLayout = new VerticalLayout();
	
	hsplit.setFirstComponent(leftLayout);
	hsplit.setSecondComponent(rightLayout);
	
	Label usersLabel = new Label("Users");
	Label permissionsLabel = new Label("Permissions");
	leftLayout.addComponent(usersLabel);
	leftLayout.addComponent(users);
	rightLayout.addComponent(permissionsLabel);
	rightLayout.addComponent(permissions);
	usersLabel.setStyleName("loginName");
	permissionsLabel.setStyleName("loginName");
	
	UserDAO uDal = new UserDAO();
	List<User> allUsers = uDal.getAllUsers();
	for (User i: allUsers){
	    users.addItem(i.getUsername());
	}
	uDal.closeSession();
	RightDAO rDal = new RightDAO();
	List<Right> allRights = rDal.getAllRights();
	for (Right i: allRights){
	    permissions.addItem(i.getPermission());
	}
	rDal.closeSession();
	permissions.setStyleName("v-select-optiongroup-horizontalgroup");
	permissions.setImmediate(true);
	users.setImmediate(true);
	users.addListener( new Property.ValueChangeListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		// first we need to unselect all
		selectedUser = users.getValue().toString();
		for (Object item: permissions.getItemIds()){
		    permissions.unselect(item);
		}
		RightDAO rDal = new RightDAO();
		List<Right> userPerms = rDal.getRightsByUser(selectedUser); 
		for (Right i: userPerms){
		    permissions.select(i.getPermission());
		}
		rDal.closeSession();
	    }
	});
	Button submitB = new Button("Submit changes");
	header.addComponent(submitB);
	submitB.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		if (!(selectedUser == null)){
		    if (!selectedUser.equals("admin")){
			ArrayList<String> selection = new ArrayList<>();
			for (Object i: (Collection)permissions.getValue()){
			    selection.add(i.toString());
			}				
			if (selection.size() != 0){
			    CommonPermissionsBroker pb = new CommonPermissionsBroker();
			    pb.changePermissionToUser(selectedUser, selection);
			    Notification.show("User permissions successfully updated");
			    init();
			}else Notification.show("Failed to update user persmissions","At least one permission has to be granted",Notification.TYPE_ERROR_MESSAGE);		
		    }else Notification.show("Failed to update user permissions","Admin user permissions can't be editted",Notification.TYPE_ERROR_MESSAGE);
		}else Notification.show("Failed to upgrade user permissions","You have to select a user first",Notification.TYPE_ERROR_MESSAGE);
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
