package com.example.groracle.Views;

import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.RoleDAO;
import com.example.groracle.DAL.DAO.UserDAO;
import com.example.groracle.Entity.Role;
import com.example.groracle.Entity.User;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.OptionGroup;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of editing users' role belonging
 * @author tdosek
 * @version 1.0
 */
public class EditUserRoleView extends Panel implements View {
    public static final String NAME = "EditUserRole";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    // Initialization of variables for the view
    private IndexedContainer userlist = null;
    private OptionGroup roles = new OptionGroup();
    private Button changeRole = new Button("Change role");
    private static String username = "User name";
    private static String email = "E-mail";
    private static String description = "Description";
    private static final String[] userFieldNames = new String[] {username,email,description};
    private Table userList = new Table();  
    //	Variables for storing pattern selected in UI
    private String selectedUsername = null;
    private Object lineId = null;
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public EditUserRoleView(Navigator navigator){
	this.navigator = navigator; 
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    protected void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	userlist = createUserlistDatasource();
	initListLayout();
	initList();
	
	HorizontalLayout roleL = new HorizontalLayout();
	header.addComponent(roleL);
	roleL.addComponent(roles);
	RoleDAO rDal = new RoleDAO();
	List<Role> allRoles = rDal.getAllRoles();
	for (Role i: allRoles){
	    roles.addItem(i.getRole());
	}
	rDal.closeSession();
	Button updateRole = new Button("Update role");
	header.addComponent(updateRole);
	
	updateRole.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		if (selectedUsername.equals("admin")){
		    Notification.show("Failed to update user role","Admin role cannot be swapped",Notification.TYPE_ERROR_MESSAGE);		    
		}else{
		    UserDAO uDal = new UserDAO();
		    User editedUser = uDal.findUser(selectedUsername);
		    editedUser.setRole(roles.getValue().toString());
		    uDal.updateUser(editedUser);
		    uDal.closeSession();
		    Notification.show("User role successfully updated");
		}	
	    }
	});
    }
    /**
     * Method for initialization of Table layout
     */
    private void initListLayout(){
	userList.setStyleName("v-table-strong");
	final VerticalLayout leftLayout = new VerticalLayout();
	header.addComponent(leftLayout);
	    
	leftLayout.addComponent(userList);
	userList.setHeight("100%");
	userList.setWidth("100%");
	leftLayout.setSizeFull();
	leftLayout.setExpandRatio(userList, 1);
    }
    /**
     * Method for initialization Table and its connection to relevant datasource
     */
    private void initList(){
	userList.setContainerDataSource(userlist);
	userList.setVisibleColumns(userFieldNames);
	userList.setSelectable(true);
	userList.setImmediate(true);
	userList.addValueChangeListener(new Property.ValueChangeListener() {
	        
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.data.Property.ValueChangeListener#valueChange(com.vaadin.data.Property.ValueChangeEvent)
	     */
	    public void valueChange(ValueChangeEvent event) {
		lineId = userList.getValue();
	    	if (lineId != null){
	    	    selectedUsername = userList.getItem(lineId).getItemProperty(username).getValue().toString();
	    	    UserDAO uDal = new UserDAO();
	    	    User selUser = uDal.findUser(selectedUsername);
	    	    uDal.closeSession();
	    	    roles.select(selUser.getRole());
	 	}	
	    }
	});
	userList.setBuffered(false);
    }
    /**
     * Method for intiialization of a data source for given table
     * @return data source
     */
    private static IndexedContainer createUserlistDatasource(){
	    // 	Loading database values into an Indexed container for further usage by UI.
		IndexedContainer ic = new IndexedContainer();
		for (String p : userFieldNames){
		    ic.addContainerProperty(p, String.class, "");
		}
		UserDAO uDal = new UserDAO();
		List<User> allUsers = uDal.getAllUsers();
		for (User i: allUsers){
		    Object id = ic.addItem();
		    String userNameAdd = i.getUsername();
		    String emailAdd = i.getEmail();
		    String descAdd = i.getDescritpion();
		    ic.getContainerProperty(id, username).setValue(userNameAdd);
		    ic.getContainerProperty(id, email).setValue(emailAdd);
		    ic.getContainerProperty(id, description).setValue(descAdd);
		}
		uDal.closeSession();
		return ic;
	    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
