package com.example.groracle.Views;

import com.example.groracle.NavigatorUI;
import com.example.groracle.Brokers.PasswordHandler;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.server.ClassResource;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Image;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of Login mechanisms
 * @author tdosek
 * @version 1.0
 */
public class LoginView extends Panel implements View {
    public static final String NAME = "Login";
    // Initializing variables for menu
    private VerticalLayout header = new VerticalLayout();
    private HorizontalLayout imgPanel = new HorizontalLayout();
    private Image headerImg = new Image(null, new ClassResource("groracle.gif"));
    private Label headerLabel = new Label("Groracle... The source of wisdom for your Grok",Label.CONTENT_RAW);
    /**
     * Constructor for class LoginView
     * @param navigator Navigator class handling whole application navigation
     * @param fragmentAndParameters Link fragment pointing to a class that is called via static link representation
     */
    public LoginView(final Navigator navigator,
	    final String fragmentAndParameters) {
		initHeader();
		// Show required login fields
		VerticalLayout editor = new VerticalLayout();
		editor.setStyleName("loginEditor");
		header.addComponent(editor);
		
		final Label usernameLabel = new Label("Username");
		usernameLabel.setStyleName("loginName");
		editor.addComponent(usernameLabel);
		
	   	final TextField username = new TextField("");
	   	username.setStyleName("loginUserText");
	   	editor.addComponent(username);

	   	final Label passwordLabel = new Label("Password");
		passwordLabel.setStyleName("loginPass");
		editor.addComponent(passwordLabel);
	   	
	   	final PasswordField password = new PasswordField("");
	   	password.setStyleName("loginPassText");
	   	editor.addComponent(password);

	   	final Button loginButton = new Button("Log me in", new Button.ClickListener() {
	   	/*
	   	 *     (non-Javadoc)
	   	 * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	   	 */
                public void buttonClick(ClickEvent event) {
                    PasswordHandler ph = new PasswordHandler();
                    if(ph.validatePassword(username.getValue(), password.getValue())){
                	((NavigatorUI)UI.getCurrent()).setLoggedInUser(username.getValue());
                	Notification.show("Welcome "+username.getValue()+"!");
                    }else{
                	((NavigatorUI)UI.getCurrent()).setLoggedInUser(null);
                    }                    
                    Object user = ((NavigatorUI)UI.getCurrent()).getLoggedInUser();
                    if (user == null){
                	Notification.show("Sorry.", "Password or username invalid", Notification.TYPE_ERROR_MESSAGE);
                    }else {
                	// navigate back to the intended place
                        navigator.navigateTo(fragmentAndParameters);
                    }                
                }
	   	});
	   	loginButton.setStyleName("loginButton");
	   	editor.addComponent(loginButton);
    	}
    	/**
    	 * Method displaying header of the application instead of menu
    	 */
       	private void initHeader(){
		// Display header
		setContent(header);	    
		header.addComponent(imgPanel);
		header.setWidth("100%");
		header.setHeight("100%");
		imgPanel.setStyleName("backColor");
		imgPanel.setWidth("100%");
		header.setWidth("100%");
		imgPanel.addComponent(headerImg);
		headerImg.setWidth("150px");
		headerImg.setHeight("113px");
		imgPanel.addComponent(headerLabel);
		headerLabel.setStyleName("headerLabel");
       	}
        @Override
        /*
         * (non-Javadoc)
         * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
         */
        public void enter(ViewChangeEvent event) {
            // No need to use this method here. 
        }
    }

