package com.example.groracle.Views;

import java.util.List;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.InputDAO;
import com.example.groracle.DAL.DAO.PatternBlacklistDAO;
import com.example.groracle.Entity.Input;
import com.example.groracle.Entity.PatternBlacklist;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.fieldgroup.FieldGroup;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of MainView with unconfirmed patterns
 * @author tdosek
 * @version 1.0
 */
public class MainView extends Panel implements View {
    public static final String NAME = "";
    private Navigator navigator = null;
    // Initializing variables for basic layout of pattern matcher
    private Table lineList = new Table();
    private Table detailList = new Table();
    private Button confirmButton = new Button("Confirm the pattern and hide it and similar lines");
    private Button blacklistPatternButton = new Button("Blacklist the pattern and similar lines");
    private FormLayout editorLayout = new FormLayout();
    private FieldGroup editorFields = new FieldGroup();
    private IndexedContainer lineContainer = createDatasource();
    private IndexedContainer details = null;
    private static final String line = "Unmatched line";
    private static final String pattern = "Suggested pattern";
    private static final String date = "Date";
    private static final String host = "Host";
    private static final String[] fieldNames = new String[] { date, host, line, pattern };
    // Variables for storing pattern selected in UI
    private static String selectedPattern = null;
    private Object lineId = null;
    // Initializing variables for menu
    private VerticalLayout header = new VerticalLayout();
    /**
     * Constructor for the class    
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public MainView(final Navigator navigator) {
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	selectedPattern = null;
	lineId = null;
	lineContainer = createDatasource();
	initPatternLayout();
	initEditor();
	initLineList();
    }
    /**
     * Method initialization of layout for table containing patterns
     */
    private void initPatternLayout(){	 
	lineList.setStyleName("v-table-strong");
	detailList.setStyleName("v-table-strong");
	final VerticalLayout leftLayout = new VerticalLayout();
	header.addComponent(leftLayout);	    
	header.addComponent(editorLayout);
  
	leftLayout.addComponent(lineList);
	lineList.setHeight("100%");
	leftLayout.setSizeFull();
	leftLayout.setExpandRatio(lineList, 1);
	    
	lineList.setSizeFull();
	    
	editorLayout.setMargin(true);
	editorLayout.setVisible(true);
    }
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	if (editorLayout.getComponentCount() == 0){
	    for (String fieldName : new String[]{pattern}){
		TextField field = new TextField(fieldName);
		editorLayout.addComponent(field);
		if (fieldName != pattern) field.setReadOnly(true);
		field.setWidth("100%");
		field.setImmediate(true);
		editorFields.bind(field, fieldName);
	    }
	    final TextField patternNameT = new TextField("Name of pattern");
	    editorLayout.addComponent(patternNameT);
	    patternNameT.setWidth("100%");
	    editorLayout.addComponent(confirmButton);
	    confirmButton.addClickListener(new ClickListener() {
	        
	        @Override
	        /*
	         * (non-Javadoc)
	         * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	         */
	        public void buttonClick(ClickEvent event) {
	            if (selectedPattern != null){
	        	PatternBlacklistDAO pbDal = new PatternBlacklistDAO();
	        	PatternBlacklist newPattern = new PatternBlacklist();
	        	newPattern.setPattern(selectedPattern);
	        	newPattern.setConfirmed(true);
	        	newPattern.setName(patternNameT.getValue());
	        	pbDal.newPatternBlacklist(newPattern);
	        	pbDal.closeSession();	        	
	        	Notification.show("Pattern confirmed and hidden from UI");
	        	navigator.navigateTo(MainView.NAME);
	        	patternNameT.setValue("");
		    }else Notification.show("Failed to confirm pattern","No pattern was selected",Notification.TYPE_ERROR_MESSAGE);
	            editorFields.getItemDataSource().getItemProperty(pattern).setValue("");
	            lineList.select(1);
	        }
	    });
	    editorLayout.addComponent(blacklistPatternButton);
	    editorFields.setBuffered(false);
	    blacklistPatternButton.addClickListener(new ClickListener() {
	    
		@Override
		/*
		 * (non-Javadoc)
		 * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
		 */
		public void buttonClick(ClickEvent event) {
		    if (selectedPattern != null){
			PatternBlacklistDAO pbDal = new PatternBlacklistDAO();
	        	PatternBlacklist newPattern = new PatternBlacklist();
	        	newPattern.setPattern(selectedPattern);
	        	newPattern.setConfirmed(false);
	        	newPattern.setName(patternNameT.getValue());
	        	pbDal.newPatternBlacklist(newPattern);
	        	pbDal.closeSession();
			Notification.show("Pattern added to blacklist");
			patternNameT.setValue("");
			navigator.navigateTo(MainView.NAME);
		    }else Notification.show("Failed to blacklist pattern","No pattern was selected",Notification.TYPE_ERROR_MESSAGE);
		    editorFields.getItemDataSource().getItemProperty(pattern).setValue("");
		    lineList.select(1);
		}
	    });
    	}
    }
    /**
     * Method for initialization of pattern table and its connection to relevant data source 
     */
    private void initLineList(){
	lineList.setContainerDataSource(lineContainer);
	lineList.setVisibleColumns(new String[]{pattern});
	lineList.setSelectable(true);
	lineList.setImmediate(true);
	   
	lineList.addValueChangeListener(new Property.ValueChangeListener() {      
	        @Override
	        public void valueChange(ValueChangeEvent event) {
	            editorLayout.removeComponent(detailList);
	            lineId = lineList.getValue();
	            if (lineId != null){
	        	editorFields.setItemDataSource(lineList.getItem(lineId));
	    	    	selectedPattern = editorFields.getItemDataSource().getItemProperty(pattern).getValue().toString();
	    		editorLayout.setVisible(lineId != null);
	    		
	    		details = null;
	    		details = createDetailsDatasource();
	    		detailList.setContainerDataSource(details);
	    		detailList.setWidth("100%");
	    		detailList.setVisibleColumns(new String[]{date,host,line});
	    		detailList.setSelectable(false);
	    		editorLayout.addComponent(detailList);
	            }	    	
	        }
	});
    }
    /**
     * Method creating datasource for main table of the view
     * @return data source
     */
    private static IndexedContainer createDatasource(){
	// Loading database values into an Indexed container for further usage by UI.
	IndexedContainer ic = new IndexedContainer();
	for (String p : fieldNames){
	    ic.addContainerProperty(p, String.class, "");
	}
	InputDAO iDal = new InputDAO();
	List<Input> allNonBlacklisted = iDal.getNonBlacklistedGrouped();
	for (Input i: allNonBlacklisted){
	    Object id  = ic.addItem();
	    String inPattern = i.getPattern();
	    ic.getContainerProperty(id, pattern).setValue(inPattern);		
	}
	iDal.closeSession();
	return ic;
    }
    /**
     * Method initializing data source for details displaying
     * @return data source
     */
    private static IndexedContainer createDetailsDatasource(){
	// Loading database values into an Indexed container for further usage by UI.
	IndexedContainer ic = new IndexedContainer();
	for (String p : fieldNames){
	    ic.addContainerProperty(p, String.class, "");
	}
	InputDAO iDal = new InputDAO();
	List<Input> allInput = iDal.getAllInput();
	for (Input i: allInput){
	    Object id  = ic.addItem();
	    String inPattern = i.getPattern();
	    String inLine = i.getLine();
	    String inHost = i.getHost();
	    String inDate = i.getDate();
	    ic.getContainerProperty(id, pattern).setValue(inPattern);
	    ic.getContainerProperty(id, line).setValue(inLine);
	    ic.getContainerProperty(id, host).setValue(inHost);
	    ic.getContainerProperty(id, date).setValue(inDate);
	}
	iDal.closeSession();
	return ic;
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();
    }
}