package com.example.groracle.Views;

import com.example.groracle.Menu;
import com.example.groracle.DAL.DAO.HostDAO;
import com.example.groracle.Entity.Host;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of functionality adding host
 * @author tdosek
 * @version 1.0
 */
public class NewHostView extends Panel implements View{
    public static final String NAME = "NewHost";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public NewHostView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	// Initialize relevant fields
	VerticalLayout editor = new VerticalLayout();
	editor.setStyleName("loginEditor");
	header.addComponent(editor);
	
	Label hostL = new Label("Host name");
	editor.addComponent(hostL);
	final TextField hostF = new TextField("");
	editor.addComponent(hostF);
	hostL.setStyleName("loginName");
	hostF.setStyleName("loginUserText");
	
	Label ipL = new Label("Host IP address");
	editor.addComponent(ipL);
	final TextField ipT = new TextField();
	editor.addComponent(ipT);
	ipL.setStyleName("loginName");
	ipT.setStyleName("loginUserText");
	
	Label descL = new Label("Description");
	editor.addComponent(descL);
	final TextArea descT = new TextArea();
	editor.addComponent(descT);
	descL.setStyleName("loginName");
	descT.setStyleName("loginUserText");
	
	Button createHostB = new Button("Create host");
	createHostB.setStyleName("loginButton");
	editor.addComponent(createHostB);
	// Process data and store them
	createHostB.addClickListener(new Button.ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		if (!(hostF.getValue()==null) && !(hostF.getValue()=="") && !(ipT.getValue()=="") && !(ipT.getValue()==null)){
		    Host newHost = new Host();
		    newHost.setHost(hostF.getValue());
		    newHost.setDescription(descT.getValue());
		    newHost.setIp(ipT.getValue());
		    HostDAO hostDal = new HostDAO();
		    if(hostDal.newHost(newHost)){
			hostF.setValue("");
			descT.setValue("");
			ipT.setValue("");
			Notification.show("Host successfully added");
			init();
		    }else Notification.show("Failed to create host","Hostname already exists",Notification.TYPE_ERROR_MESSAGE);
		    hostDal.closeSession();
		}else Notification.show("Failed to create host","Both IP and host name field have to be filled in",Notification.TYPE_ERROR_MESSAGE);		
	    }
	});
    }  
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
