package com.example.groracle.Views;

import java.util.ArrayList;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.CommonPermissionsBroker;
import com.example.groracle.DAL.DAO.RoleDAO;
import com.example.groracle.Entity.Role;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Button.ClickEvent;
/**
 * Class representing UI implementation of adding new role to the system
 * @author tdosek
 * @version 1.0
 */
public class NewRoleView extends Panel implements View{
    public static final String NAME = "NewRole";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public NewRoleView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    protected void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    /**
     * Initialization method for UI layout of the editor
     */
    protected void initEditor(){
	VerticalLayout editor = new VerticalLayout();
	editor.setStyleName("loginEditor");
	header.addComponent(editor);
	
	Label roleL = new Label("Role name");
	editor.addComponent(roleL);
	final TextField roleF = new TextField("");
	editor.addComponent(roleF);
	roleL.setStyleName("loginName");
	roleF.setStyleName("loginUserText");
	
	Button createRoleB = new Button("Create role");
	createRoleB.setStyleName("loginButton");
	editor.addComponent(createRoleB);
	
	createRoleB.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		String roleName = roleF.getValue();
		if ((!roleName.equals(null)) && (!roleName.equals(""))){
		    if (!roleName.contains(";")){
			Role newRole = new Role();
			newRole.setRole(roleName);
			newRole.setGroups(null);
			RoleDAO roleDal = new RoleDAO();  
			if(roleDal.newRole(newRole)){
			    ArrayList<String> newPermissions = new ArrayList<>();
			    newPermissions.add("EditPattern");
			    CommonPermissionsBroker pb = new CommonPermissionsBroker();
			    pb.changePermissionToRole(roleName, newPermissions);
			    init();
			    Notification.show("Role succesfully created");
			}else Notification.show("Failed to create role","Role name already exists",Notification.TYPE_ERROR_MESSAGE);
			roleDal.closeSession();
		    }else Notification.show("Failed to create role","Role name can't contain ; sign",Notification.TYPE_ERROR_MESSAGE);
		}else Notification.show("Failed to create role","Role name can't be empty",Notification.TYPE_ERROR_MESSAGE);
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();
	
    }

}
