package com.example.groracle.Views;

import com.example.groracle.Menu;
import com.example.groracle.NavigatorUI;
import com.example.groracle.Brokers.PasswordHandler;
import com.example.groracle.DAL.DAO.UserDAO;
import com.example.groracle.Entity.User;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PasswordField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation of password reseting for user
 * @author tdosek
 * @version 1.0
 */
public class ResetPasswordView extends Panel implements View {
    public static final String NAME = "ResetPassword";
    private Navigator navigator = null;
    private VerticalLayout header = new VerticalLayout();
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public ResetPasswordView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initEditor();
    }
    /**
     * Initialization method for UI layout of the editor
     */
    private void initEditor(){
	// Display relevant fields
	VerticalLayout editor = new VerticalLayout();
	editor.setStyleName("loginEditor");
	header.addComponent(editor);
	
	Label oldLabel = new Label("Old password*");
	editor.addComponent(oldLabel);
	final PasswordField oldPass = new PasswordField("");
	editor.addComponent(oldPass);
	oldLabel.setStyleName("loginName");
	oldPass.setStyleName("loginUserText");
	
	Label newLable = new Label("New password*");
	editor.addComponent(newLable);
	final PasswordField newPass = new PasswordField("");
	editor.addComponent(newPass);
	newLable.setStyleName("loginName");
	newPass.setStyleName("loginUserText");
	
	Label newConfLabel = new Label("Confirm password*");
	editor.addComponent(newConfLabel);
	final PasswordField newConfPass = new PasswordField("");
	editor.addComponent(newConfPass);
	newConfLabel.setStyleName("loginName");
	newConfPass.setStyleName("loginUserText");
	
	Button changePass = new Button("Change password");
	changePass.setStyleName("loginButton");
	editor.addComponent(changePass);
	// Execute change and handle it
	changePass.addClickListener(new ClickListener() {
	    
	    @Override
	    /*
	     * (non-Javadoc)
	     * @see com.vaadin.ui.Button.ClickListener#buttonClick(com.vaadin.ui.Button.ClickEvent)
	     */
	    public void buttonClick(ClickEvent event) {
		PasswordHandler ph = new PasswordHandler();
		String user = ((NavigatorUI)UI.getCurrent()).getLoggedInUser();
		String oldPassword = oldPass.getValue();
		String newPassword = newPass.getValue();
		String newConfirmedPass = newConfPass.getValue();
		if (ph.validatePassword(user, oldPassword)){
		    if(newPassword.equals(newConfirmedPass)){
			String email = null;
			UserDAO uDal = new UserDAO();
			User resetUser = uDal.findUser(user);
			email = resetUser.getEmail();
			if (!email.equals(null)){
			    ph.setNewPassword(user, newPassword, email);
			    Notification.show("Password successfully changed");
			}else{
			    Notification.show("Failed to change password","No e-mail found for given user", Notification.TYPE_ERROR_MESSAGE);
			}
			uDal.closeSession();
		    }else{
			Notification.show("Failed to change password","New password and its confirmation do not match", Notification.TYPE_ERROR_MESSAGE);
		    }
		}else{
		    Notification.show("Failed to change password","Old password is invalid", Notification.TYPE_ERROR_MESSAGE);
		}
	    }
	});
    }
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();
    }

}
