package com.example.groracle.Views;

import com.example.groracle.Menu;
import com.example.groracle.Brokers.Statistics;
import com.vaadin.navigator.Navigator;
import com.vaadin.navigator.View;
import com.vaadin.navigator.ViewChangeListener.ViewChangeEvent;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
/**
 * Class representing UI implementation for showing various statistics
 * @author tdosek
 * @version 1.0
 */
public class StatisticsView extends Panel implements View{
    public static final String NAME = "Statistics";
    private Navigator navigator = null;
    // Initializing variables for view
    private VerticalLayout header = new VerticalLayout();
    /**
     * Constructor of the class
     * @param navigator variable representing UI navigator class which handles UI element switching 
     */
    public StatisticsView(Navigator navigator){
	this.navigator = navigator;
    }
    /**
     * Initialization method which handles composition of UI for given class
     */
    private void init(){
	setContent(header);
	header.removeAllComponents();
	Menu menu = new Menu(navigator);
	menu.initMenu();
	header.addComponent(menu);
	initStats();
    }
    /**
     * Method initializing statistics layout and display
     */
    private void initStats(){
	VerticalLayout statLayout = new VerticalLayout();
	header.addComponent(statLayout);
	statLayout.setImmediate(true);
	statLayout.setWidth("100%");
	statLayout.setHeight("100%");
	// Generate statistics 
	Statistics statsLo = new Statistics("LoadInTime");
	Statistics statsLi = new Statistics("LinesPerPattern");
	Statistics statsHo = new Statistics("LinesPerHost");
	// Display charts in this view with relevant labeling
	Label statLoLabel = new Label("Load in time:");
	statLoLabel.setStyleName("loginName");
	statLayout.addComponent(statLoLabel);
	statLayout.addComponent(statsLo.drawStatistics());
	
	Label statLiLabel = new Label("Amount of lines per pattern:");
	statLiLabel.setStyleName("loginName");
	statLayout.addComponent(statLiLabel);
	statLayout.addComponent(statsLi.drawStatistics());
	
	Label statHoLabel = new Label("Amount of lines per host:");
	statHoLabel.setStyleName("loginName");
	statLayout.addComponent(statHoLabel);
	statLayout.addComponent(statsHo.drawStatistics());
    }
    
    @Override
    /*
     * (non-Javadoc)
     * @see com.vaadin.navigator.View#enter(com.vaadin.navigator.ViewChangeListener.ViewChangeEvent)
     */
    public void enter(ViewChangeEvent event) {
	init();	
    }

}
